/*******************************************************************************
@ddblock_begin copyright

Copyright (c) 1999-2013
Maryland DSPCAD Research Group, The University of Maryland at College Park 

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF MARYLAND BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF MARYLAND HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF MARYLAND SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
MARYLAND HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

@ddblock_end copyright
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "lide_c_actor.h"
#include "lide_c_fifo.h"
#include "lide_c_bmp_file_read_halo.h"
#include "lide_c_bmp_file_write.h"
#include "lide_c_file_source_fcv.h"
#include "lide_c_gfilter.h"
#include "lide_c_invert.h"
#include "lide_c_t.h"
#include "lide_c_util.h"
#include "lide_c_bmp.h"

#define BUFFER_CAPACITY 1024

/* An enumeration of the actors in this application. */
#define ACTOR_BMP_SOURCE 0
#define ACTOR_COEF_SOURCE 1
#define ACTOR_BMP_INVERT 2 
#define ACTOR_BMP_FILTER 3 
#define ACTOR_BMP_SINK 4
#define ACTOR_T 5

/* The total number of actors in the application. */
#define ACTOR_COUNT 6

/* 
    Usage: driver.exe coef_file in_img_file out_img_file
*/
int main(int argc, char **argv) {
    FILE *in_file = NULL; 
    FILE *out_file = NULL; 
    FILE *coef_file = NULL;

    lide_c_actor_context_type *actors[ACTOR_COUNT];
    lide_c_bmp_file_read_halo_context_type *in_source = NULL;
        
    lide_c_fifo_pointer fifo0 = NULL;
    lide_c_fifo_pointer fifo1 = NULL;
    lide_c_fifo_pointer fifo2 = NULL;
    lide_c_fifo_pointer fifo_newrow = NULL;
    lide_c_fifo_pointer fifo_bmpinfo = NULL;
    lide_c_fifo_pointer fifo_coef = NULL;
    lide_c_fifo_pointer fifo_coef1 = NULL;
    lide_c_fifo_pointer fifo_coef2 = NULL;

    int token_size = 0;
    int i = 0;
    int arg_count = 4; /*driver coef in out */ 
        
    /* actor descriptors (for diagnostic output) */
    char *descriptors[ACTOR_COUNT] = {"img_source", "coef_source", "invert",
				      "gfilter", "sink", "T"};

    /* Check program usage. */
    if (argc != arg_count) {
        fprintf(stderr, "driver.exe error: arg count");
        exit(1);
    }   

    /* Open the parameters file(s) */
    i = 1;

    /* Open the input and output file(s). */
    coef_file = lide_c_util_fopen(argv[i++], "r"); 
    in_file = lide_c_util_fopen(argv[i++], "rb"); 
    out_file = lide_c_util_fopen(argv[i++], "wb"); 

    /* Create the buffers. */
    token_size = sizeof(float*);
    fifo0 = lide_c_fifo_new(BUFFER_CAPACITY, token_size);
    fifo1 = lide_c_fifo_new(BUFFER_CAPACITY, token_size);
    fifo2 = lide_c_fifo_new(BUFFER_CAPACITY, token_size);
    fifo_newrow = lide_c_fifo_new(BUFFER_CAPACITY, sizeof(int));
    fifo_bmpinfo = lide_c_fifo_new(1, sizeof(bmp_file_info));
    fifo_coef = lide_c_fifo_new(1, sizeof(float*));
    fifo_coef1 = lide_c_fifo_new(1, sizeof(float*));
    fifo_coef2 = lide_c_fifo_new(1, sizeof(float*));
    
    /* Create and connect the actors. */
    actors[ACTOR_BMP_SOURCE] = (lide_c_actor_context_type*)
            (lide_c_bmp_file_read_halo_new(in_file, 128, 128, fifo_coef2, fifo0,
            fifo_newrow,fifo_bmpinfo));

    actors[ACTOR_COEF_SOURCE] = 
            (lide_c_actor_context_type *)(lide_c_file_source_fcv_new(coef_file,
            fifo_coef));

    actors[ACTOR_T] =
            (lide_c_actor_context_type*)(lide_c_t_new(sizeof(float*),fifo_coef,
            fifo_coef1, fifo_coef2));

    actors[ACTOR_BMP_FILTER] = (lide_c_actor_context_type *)
            (lide_c_gfilter_new(128, 128, fifo_coef1, fifo0, fifo1));

    actors[ACTOR_BMP_INVERT] = (lide_c_actor_context_type *)
            (lide_c_invert_new(128, 128, fifo1, fifo2));

    actors[ACTOR_BMP_SINK] = 
            (lide_c_actor_context_type *)(lide_c_bmp_file_write_new(out_file, 
            128, 128, fifo2, fifo_newrow, fifo_bmpinfo));

    in_source = (lide_c_bmp_file_read_halo_context_type *)
            (actors[ACTOR_BMP_SOURCE]);

    /* A simple scheduler: keep executing actors until one of the
       sources has finished, and there are no more result tokens
       waiting to be written to the output file. 
    */
    lide_c_util_simple_scheduler(actors, ACTOR_COUNT, descriptors);

    fclose(out_file);

    /* Normal termination. */
    return 0;
}
